import logging
from google.cloud import pubsub_v1
from google.oauth2 import service_account
from google.auth import default

logger = logging.getLogger(__name__)


class PubSubClient:
    def __init__(
        self,
        project_id: str,
        topic: str,
        keyfile_path: None | str = None,
    ):
        logger.info(f"Creating publisher client..")

        if keyfile_path:
            logger.info("Creating credentials with SA keyfile..")
            self._credentials = service_account.Credentials.from_service_account_file(
                keyfile_path
            )
        else:
            # https://cloud.google.com/docs/authentication/provide-credentials-adc#how-to
            logger.info(
                "Creating credentials with Application Default Credentials (ADC).."
            )
            self._credentials, _ = default()

        self._publisher_client = pubsub_v1.PublisherClient(
            credentials=self._credentials
        )

        self._topic_path = self._publisher_client.topic_path(
            project_id,
            topic,
        )
        logger.debug(f"{self._topic_path=}")

    def publish(self, msg: str | bytes) -> None:
        # Convert the message to bytes.
        if isinstance(msg, str):
            msg_bytes: bytes = msg.encode("utf-8")
        else:
            msg_bytes = msg

        # Publish the message to topic.
        future = self._publisher_client.publish(self._topic_path, data=msg_bytes)

        # block until the message has been published successfully
        message_id = future.result()

        logger.info(f"Message '{msg}' published. Message ID {message_id}")



import os
from datetime import datetime
import threading
import logging.config
from pythonjsonlogger import jsonlogger


def get_json_formatter(prefix: str):
    class CustomJsonFormatter(jsonlogger.JsonFormatter):
        # override
        def add_fields(self, log_record, record, message_dict):
            super().add_fields(log_record, record, message_dict)
            log_record["datetime"] = datetime.utcnow().isoformat()
            log_record["level"] = record.levelname
            log_record["logger_name"] = record.name
            log_record["message"] = f"{prefix}{record.getMessage()}"
            log_record["process"] = os.getpid()
            log_record["thread"] = threading.current_thread().name
            log_record["filename"] = record.filename
            log_record["line_number"] = record.lineno

    return CustomJsonFormatter


def set_logging_config(
    prefix: str = "", json_log: bool = False, log_level: str = "INFO"
):
    logging.config.dictConfig(
        {
            "version": 1,
            "disable_existing_loggers": False,
            "formatters": {
                "json": {
                    "()": get_json_formatter(prefix=prefix),
                    "format": "%(severity)s %(asctime)s %(name)s %(funcName)s %(pathname)s %(lineno)s %(message)s",
                },
                "detail": {
                    "format": "{asctime} {process} {processName} {name} [{levelname}] {message}",
                    "style": "{",
                },
            },
            "handlers": {
                "console": {
                    "class": "logging.StreamHandler",
                    "formatter": "json" if json_log else "detail",
                },
            },
            "loggers": {
                "": {"handlers": ["console"], "level": log_level},
            },
        }
    )



import orjson
import base64
from dataclasses import dataclass, asdict
from enum import Enum, auto, Flag


@dataclass
class ReportJobEvent:
    report_jod_id: int
    symbol: str
    past_months: int


@dataclass
class StockAnalysisEvent:
    symbol: str
    start: str
    end: str
    email: str

    @property
    def get_key(self):
        return f"{self.symbol}_{self.start}_{self.end}"


@dataclass
class NotificationEvent:
    class Types(Flag):
        EMAIL = auto()
        SMS = auto()
        TELEGRAM = auto()

    user_id: str
    title: str
    msg: str
    type: Types

    @classmethod
    def from_dict(cls, d: dict):
        return cls(
            user_id=d["user_id"],
            title=d["title"],
            msg=d["msg"],
            type=cls.Types(d["type"]),
        )

    @classmethod
    def from_base64_str(cls, s: str):
        base64_decoded_bytes: bytes = base64.b64decode(s)
        d: dict = orjson.loads(base64_decoded_bytes)
        return cls(
            user_id=d["user_id"],
            title=d["title"],
            msg=d["msg"],
            type=cls.Types(d["type"]),
        )

    def to_bytes(self) -> bytes:
        return orjson.dumps(asdict(self), option=orjson.OPT_NAIVE_UTC)
